'use strict';

exports.__esModule = true;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.toInstallOptions = toInstallOptions;
exports.install = install;

var _Project = require('../Project');

var _Project2 = _interopRequireDefault(_Project);

var _options = require('../utils/options');

var options = _interopRequireWildcard(_options);

var _processes = require('../utils/processes');

var processes = _interopRequireWildcard(_processes);

var _fs = require('../utils/fs');

var fs = _interopRequireWildcard(_fs);

var _path = require('path');

var path = _interopRequireWildcard(_path);

var _logger = require('../utils/logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('../utils/messages');

var messages = _interopRequireWildcard(_messages);

var _validateProject = require('../utils/validateProject');

var _validateProject2 = _interopRequireDefault(_validateProject);

var _symlinkPackageDependencies = require('../utils/symlinkPackageDependencies');

var _symlinkPackageDependencies2 = _interopRequireDefault(_symlinkPackageDependencies);

var _yarn = require('../utils/yarn');

var yarn = _interopRequireWildcard(_yarn);

var _pathIsInside = require('path-is-inside');

var _pathIsInside2 = _interopRequireDefault(_pathIsInside);

var _errors = require('../utils/errors');

var _constants = require('../constants');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toInstallOptions(args, flags) {
  return {
    cwd: options.string(flags.cwd, 'cwd'),
    pureLockfile: options.boolean(flags.pureLockfile, 'pure-lockfile') || false
  };
}
async function install(opts) {
  let cwd = opts.cwd || process.cwd();
  let project = await _Project2.default.init(cwd);
  let workspaces = await project.getWorkspaces();
  let installFlags = [];

  if (opts.pureLockfile) installFlags.push('--pure-lockfile');

  logger.info(messages.validatingProject(), { emoji: '🔎', prefix: false });

  let projectIsValid = await (0, _validateProject2.default)(project);
  if (!projectIsValid) {
    throw new _errors.BoltError(messages.unableToInstall());
  }

  logger.info(messages.installingProjectDependencies(), {
    emoji: '📦',
    prefix: false
  });

  const yarnUserAgent = await yarn.userAgent();
  const boltUserAgent = `bolt/${_constants.BOLT_VERSION} ${yarnUserAgent}`;

  await processes.spawn('yarn', ['install', ...installFlags], {
    cwd: project.pkg.dir,
    tty: true,
    env: (0, _extends3.default)({}, process.env, { npm_config_user_agent: boltUserAgent })
  });

  logger.info(messages.linkingWorkspaceDependencies(), {
    emoji: '🔗',
    prefix: false
  });

  for (let workspace of workspaces) {
    const dependencies = workspace.pkg.getAllDependencies().keys();
    await (0, _symlinkPackageDependencies2.default)(project, workspace.pkg, dependencies);
  }

  logger.success(messages.installedAndLinkedWorkspaces(), { emoji: '💥' });
}